@@ -46,6 +46,14 @@
 /*	of taking the names literally.
 /*
 /*	This feature is available in Postfix 2.0 and later.
+/*.IP \fB-r\fR
+/*	Print records in file order, don't follow pointer records.
+/*
+/*	This feature is available in Postfix 3.7 and later.
+/* IP "\fB-s \fIoffset\fR"
+/*	Skip to the specified queue file offset.
+/*
+/*	This feature is available in Postfix 2.0 and later.
 /* .IP \fB-v\fR
 /*	Enable verbose logging for debugging purposes. Multiple \fB-v\fR
 /*	options make the software increasingly verbose.
@@ -67,6 +75,10 @@
 /* .IP "\fBconfig_directory (see 'postconf -d' output)\fR"
 /*	The default location of the Postfix main.cf and master.cf
 /*	configuration files.
+/* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
+/*	The list of environment parameters that a privileged Postfix
+/*	process will import from a non-Postfix parent process, or name=value
+/*	environment overrides.
 /* .IP "\fBqueue_directory (see 'postconf -d' output)\fR"
 /*	The location of the Postfix top-level queue directory.
 /* FILES
@@ -110,6 +122,7 @@
 #include <vstring_vstream.h>
 #include <stringops.h>
 #include <warn_stat.h>
+#include <clean_env.h>
 
 /* Global library. */
 
@@ -122,6 +135,7 @@
 #include <mail_proto.h>
 #include <is_header.h>
 #include <lex_822.h>
+#include <mail_parm_split.h>
 
 /* Application-specific. */
 
@@ -132,6 +146,7 @@
 #define PC_FLAG_PRINT_BODY	(1<<4)	/* print body records */
 #define PC_FLAG_PRINT_RTYPE_DEC	(1<<5)	/* print decimal record type */
 #define PC_FLAG_PRINT_RTYPE_SYM	(1<<6)	/* print symbolic record type */
+#define PC_FLAG_RAW		(1<<7)	/* don't follow pointers */
 
 #define PC_MASK_PRINT_TEXT	(PC_FLAG_PRINT_HEADER | PC_FLAG_PRINT_BODY)
 #define PC_MASK_PRINT_ALL	(PC_FLAG_PRINT_ENV | PC_MASK_PRINT_TEXT)
@@ -143,6 +158,8 @@
 #define PC_STATE_HEADER	1		/* primary header */
 #define PC_STATE_BODY	2		/* other */
 
+off_t   start_offset = 0;
+
 #define STR	vstring_str
 #define LEN	VSTRING_LEN
 
@@ -168,10 +185,27 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 #define TEXT_RECORD(rec_type) \
 	    (rec_type == REC_TYPE_CONT || rec_type == REC_TYPE_NORM)
 
+    /*
+     * Skip over or absorb some bytes.
+     */
+    if (start_offset > 0) {
+	if (fp == VSTREAM_IN) {
+	    for (offset = 0; offset < start_offset; offset++)
+		if (VSTREAM_GETC(fp) == VSTREAM_EOF)
+		    msg_fatal("%s: skip %ld bytes failed after %ld",
+			      VSTREAM_PATH(fp), start_offset, offset);
+	} else {
+	    if (vstream_fseek(fp, start_offset, SEEK_SET) < 0)
+		msg_fatal("%s: seek to %ld: %m",
+			  VSTREAM_PATH(fp), start_offset);
+	}
+    }
+
     /*
      * See if this is a plausible file.
      */
-    if ((ch = VSTREAM_GETC(fp)) != VSTREAM_EOF) {
+    if (start_offset == 0 && (flags & PC_FLAG_RAW) == 0
+	&& (ch = VSTREAM_GETC(fp)) != VSTREAM_EOF) {
 	if (!strchr(REC_TYPE_ENVELOPE, ch)) {
 	    msg_warn("%s: input is not a valid queue file", VSTREAM_PATH(fp));
 	    return;
@@ -182,7 +216,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
     /*
      * Other preliminaries.
      */
-    if (flags & PC_FLAG_PRINT_ENV)
+    if (start_offset == 0 && (flags & PC_FLAG_PRINT_ENV))
 	vstream_printf("*** ENVELOPE RECORDS %s ***\n",
 		       VSTREAM_PATH(fp));
     state = PC_STATE_ENV;
@@ -287,6 +321,8 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    /* Optional output. */
 	    if (flags & PC_FLAG_PRINT_ENV)
 		PRINT_MARKER(flags, fp, offset, rec_type, "MESSAGE FILE END");
+	    if (flags & PC_FLAG_RAW)
+		continue;
 	    /* Terminate the state machine. */
 	    break;
 	} else if (rec_type == REC_TYPE_PTR) {
@@ -295,7 +331,8 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    if (do_print)
 		PRINT_RECORD(flags, offset, rec_type, STR(buffer));
 	    /* Skip to the pointer's target record. */
-	    if (rec_goto(fp, STR(buffer)) == REC_TYPE_ERROR)
+	    if ((flags & PC_FLAG_RAW) == 0
+		&& rec_goto(fp, STR(buffer)) == REC_TYPE_ERROR)
 		msg_fatal("bad pointer record, or input is not seekable");
 	    continue;
 	} else if (rec_type == REC_TYPE_SIZE) {
@@ -308,7 +345,7 @@ static void postcat(VSTREAM *fp, VSTRING *buffer, int flags)
 	    } else {
 		if (sscanf(STR(buffer), "%ld %ld", &data_size, &data_offset) != 2
 		    || data_offset <= 0 || data_size <= 0)
-		    msg_fatal("invalid size record: %.100s", STR(buffer));
+		    msg_warn("invalid size record: %.100s", STR(buffer));
 		/* Optimization: skip to the message header. */
 		if ((flags & PC_FLAG_PRINT_ENV) == 0) {
 		    if (vstream_fseek(fp, data_offset, SEEK_SET) < 0)
@@ -424,6 +461,7 @@ int     main(int argc, char **argv)
     };
     char  **cpp;
     int     tries;
+    ARGV   *import_env;
 
     /*
      * Fingerprint executables and core dumps.
@@ -445,10 +483,15 @@ int     main(int argc, char **argv)
      */
     msg_vstream_init(argv[0], VSTREAM_ERR);
 
+    /*
+     * Check the Postfix library version as soon as we enable logging.
+     */
+    MAIL_VERSION_CHECK;
+
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "bc:dehoqv")) > 0) {
+    while ((ch = GETOPT(argc, argv, "bc:dehoqrs:v")) > 0) {
 	switch (ch) {
 	case 'b':
 	    flags |= PC_FLAG_PRINT_BODY;
@@ -472,6 +515,13 @@ int     main(int argc, char **argv)
 	case 'q':
 	    flags |= PC_FLAG_SEARCH_QUEUE;
 	    break;
+	case 'r':
+	    flags |= PC_FLAG_RAW;
+	    break;
+	case 's':
+	    if (!alldig(optarg) || (start_offset = atol(optarg)) <= 0)
+		msg_fatal("bad offset: %s", optarg);
+	    break;
 	case 'v':
 	    msg_verbose++;
 	    break;
@@ -486,6 +536,9 @@ int     main(int argc, char **argv)
      * Further initialization...
      */
     mail_conf_read();
+    import_env = mail_parm_split(VAR_IMPORT_ENVIRON, var_import_environ);
+    update_env(import_env->argv);
+    argv_free(import_env);
 
     /*
      * Initialize.
