@@ -88,12 +88,12 @@ static unsigned long showq_message(VSTREAM *showq_stream)
     static VSTRING *why = 0;
     long    arrival_time;
     long    message_size;
-    int     message_status;
     char   *saved_reason = mystrdup("");
     const char *show_reason;
     int     padding;
     int     showq_status;
     time_t  time_t_arrival_time;
+    int     forced_expire;
 
     /*
      * One-time initialization.
@@ -109,13 +109,15 @@ static unsigned long showq_message(VSTREAM *showq_stream)
     /*
      * Read the message properties and sender address.
      */
-    if (attr_scan(showq_stream, ATTR_FLAG_MORE | ATTR_FLAG_STRICT,
+    if (attr_scan(showq_stream, ATTR_FLAG_MORE | ATTR_FLAG_STRICT
+		  | ATTR_FLAG_PRINTABLE,
 		  RECV_ATTR_STR(MAIL_ATTR_QUEUE, queue_name),
 		  RECV_ATTR_STR(MAIL_ATTR_QUEUEID, queue_id),
 		  RECV_ATTR_LONG(MAIL_ATTR_TIME, &arrival_time),
 		  RECV_ATTR_LONG(MAIL_ATTR_SIZE, &message_size),
+		  RECV_ATTR_INT(MAIL_ATTR_FORCED_EXPIRE, &forced_expire),
 		  RECV_ATTR_STR(MAIL_ATTR_SENDER, addr),
-		  ATTR_TYPE_END) != 5)
+		  ATTR_TYPE_END) != 6)
 	msg_fatal_status(EX_SOFTWARE, "malformed showq server response");
 
     /*
@@ -123,9 +125,13 @@ static unsigned long showq_message(VSTREAM *showq_stream)
      * left-aligned, followed by other status info and the sender address
      * which is already in externalized RFC 5321 form.
      */
-    message_status = (strcmp(STR(queue_name), MAIL_QUEUE_ACTIVE) == 0 ? '*' :
-		 strcmp(STR(queue_name), MAIL_QUEUE_HOLD) == 0 ? '!' : ' ');
-    vstring_sprintf(id_status, "%s%c", STR(queue_id), message_status);
+    vstring_strcpy(id_status, STR(queue_id));
+    if (strcmp(STR(queue_name), MAIL_QUEUE_ACTIVE) == 0)
+	vstring_strcat(id_status, "*");
+    else if (strcmp(STR(queue_name), MAIL_QUEUE_HOLD) == 0)
+	vstring_strcat(id_status, "!");
+    if (forced_expire)
+	vstring_strcat(id_status, "#");
     time_t_arrival_time = arrival_time;
     vstream_printf(var_long_queue_ids ?
 		   L_SENDER_FORMAT : S_SENDER_FORMAT, STR(id_status),
@@ -138,7 +144,8 @@ static unsigned long showq_message(VSTREAM *showq_stream)
      * resynchronize.
      */
     while ((showq_status = attr_scan_more(showq_stream)) > 0) {
-	if (attr_scan(showq_stream, ATTR_FLAG_MORE | ATTR_FLAG_STRICT,
+	if (attr_scan(showq_stream, ATTR_FLAG_MORE | ATTR_FLAG_STRICT
+		      | ATTR_FLAG_PRINTABLE,
 		      RECV_ATTR_STR(MAIL_ATTR_RECIP, addr),
 		      RECV_ATTR_STR(MAIL_ATTR_WHY, why),
 		      ATTR_TYPE_END) != 2)
